/************************************************************************/
/* @file   main.c                                                       */
/* @author yangx                                                        */
/* @date   2022-4-12                                                    */
/* @brief  communicte with STM32F407 by USART                           */
/*         instruct the board to do                                     */
/*         -- set key                                                   */
/*         -- encryption by AES-128 masked code-based masking           */
/*         -- return the cyphertext                                     */
/*         note that before encryption, the key sholud be set first.    */
/*         the generator matrix is fixed as [1,0,0;91,1,0;239,0,1].     */
/************************************************************************/

#include <stm32f4xx.h>
#include <misc.h>			
#include <stm32f4xx_usart.h> 
#include <stm32f4xx_rcc.h>

#include "aes.h"
#include "comm.h"
#include "random.h"
#include "DWT_clock.h"

// Global variables
// ---------------------
// for random
uint8 randomSeed[14432] = {0}; // store all the random numbers utilized in AES-128
uint8 random1[4 * 4] = {0}; // store R1 
uint8 random2[4 * 4 * 16] = {0}; // store R2
uint8 rawRand[32] = {0};
uint8 matGByte1 = 91;
uint8 matGByte2 = 239;

// key and plaintexts
uint8 key_aes[16] = {
	0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
	0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f
};
uint8 pt_aes[16] = { 
	0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
	0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff
};
uint8 keys[16 * 11] = {0}; // keys for 11 AES rounds
uint8 maskedKeys[48 * 11] = {0}; // masked keys for 11 AES rounds

unsigned int sys_val = 0; // record the clock cycle counts

int main()
{
	int i, j;

	uint8 cmd;
	uint8 APDUcmd[5], APDUdata[16];

	uint8 ct_aes[16] = {0};

	// Communication
	Usart_Init(9600);

	// random
	initRandSrc();

	// initialize the DWT for clock cycles counting
	InitCYCCNT();
	StartCYCCNT();

	// locked loop
	// ---------------------------
	while (1)
	{

		// get the instruction
		for (i = 0; i < 2; i++)
		{
			APDUcmd[i] = Usart_GetByte();
		}
		if (APDUcmd[0] == CLA) Usart_Send(&APDUcmd[1], 1);

		switch (APDUcmd[1])
		{
		// set the key
		case INS_MAKE_AES_KEY:
			for (i = 0; i < 16; i++)
			{
				APDUdata[i] = Usart_GetByte();
				key_aes[i] = APDUdata[i];
			}
			cmd = 0x91;
			Usart_Send(&cmd, 1); // reply
			break;
		// encrypt
		case INS_ENCRYPT_AES:
			// get plainetxts from USART
			for (i = 0; i < 16; i++)
			{
				APDUdata[i] = Usart_GetByte();
				pt_aes[i] = APDUdata[i];
			}

			cmd = 0x92;
			Usart_Send(&cmd, 1); // reply

			getRand();
			rawKeyExpansion(key_aes, keys);
			encodeKeys(keys, maskedKeys);
			ResetCYCCNT();

			// AES encryption
			AES_Encrypt_GCB(ct_aes);

			sys_val = ReadCYCCNT();

			// return the cycle count
			cmd = (uint8)sys_val;
			Usart_Send(&cmd, 1);
			sys_val = sys_val >> 8;
			cmd = (uint8)(sys_val);
			Usart_Send(&cmd, 1);
			sys_val = sys_val >> 8;
			cmd = (uint8)(sys_val);
			Usart_Send(&cmd, 1);
			sys_val = sys_val >> 8;
			cmd = (uint8)(sys_val);
			Usart_Send(&cmd, 1);

			break;
		// for test
		case INS_TEST:
			getRand();
			rawKeyExpansion(key_aes, keys);
			encodeKeys(keys, maskedKeys);
			ResetCYCCNT();
			AES_Encrypt_GCB(ct_aes);
			sys_val = ReadCYCCNT();
			cmd = 0x93;
			Usart_Send(&cmd, 1);

			// return the cycle count
			cmd = (uint8)sys_val;
			Usart_Send(&cmd, 1);
			sys_val = sys_val >> 8;
			cmd = (uint8)(sys_val);
			Usart_Send(&cmd, 1);
			sys_val = sys_val >> 8;
			cmd = (uint8)(sys_val);
			Usart_Send(&cmd, 1);
			sys_val = sys_val >> 8;
			cmd = (uint8)(sys_val);
			Usart_Send(&cmd, 1);

			break;
		// get the ciphertext
		case INS_GET_RESPONSE:
			for (i = 0;i < 16;i++)
			{
				Usart_Send(&(ct_aes[i]), 1);
			}
			cmd = 0x94;
			Usart_Send(&cmd, 1); // reply
			break;
		default:
			break;
		}

	}
	

}

